package com.sun.xml.internal.ws.model.wsdl;

import com.sun.xml.internal.ws.api.model.wsdl.WSDLBoundPortType;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLService;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLExtension;
import java.util.List;
import org.xml.sax.Locator;
import com.sun.xml.internal.ws.api.model.ParameterBinding;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLMessage;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLOperation;
import java.util.Iterator;
import javax.jws.WebParam;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLPart;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLBoundOperation;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPortType;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLPort;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.HashMap;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLService;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLBoundPortType;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLPortType;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLMessage;
import javax.xml.namespace.QName;
import java.util.Map;
import com.sun.xml.internal.ws.api.model.wsdl.editable.EditableWSDLModel;

public final class WSDLModelImpl extends AbstractExtensibleImpl implements EditableWSDLModel
{
    private final Map<QName, EditableWSDLMessage> messages;
    private final Map<QName, EditableWSDLPortType> portTypes;
    private final Map<QName, EditableWSDLBoundPortType> bindings;
    private final Map<QName, EditableWSDLService> services;
    private PolicyMap policyMap;
    private final Map<QName, EditableWSDLBoundPortType> unmBindings;
    
    public WSDLModelImpl(@NotNull String systemId) {
        super(systemId, -1);
        this.messages = new HashMap();
        this.portTypes = new HashMap();
        this.bindings = new HashMap();
        this.services = new LinkedHashMap();
        this.unmBindings = Collections.unmodifiableMap(this.bindings);
    }
    
    public WSDLModelImpl() {
        super(null, -1);
        this.messages = new HashMap();
        this.portTypes = new HashMap();
        this.bindings = new HashMap();
        this.services = new LinkedHashMap();
        this.unmBindings = Collections.unmodifiableMap(this.bindings);
    }
    
    public void addMessage(EditableWSDLMessage msg) {
        this.messages.put(msg.getName(), msg);
    }
    
    public EditableWSDLMessage getMessage(QName name) {
        return (EditableWSDLMessage)this.messages.get(name);
    }
    
    public void addPortType(EditableWSDLPortType pt) {
        this.portTypes.put(pt.getName(), pt);
    }
    
    public EditableWSDLPortType getPortType(QName name) {
        return (EditableWSDLPortType)this.portTypes.get(name);
    }
    
    public void addBinding(EditableWSDLBoundPortType boundPortType) {
        assert !this.bindings.containsValue(boundPortType);
        this.bindings.put(boundPortType.getName(), boundPortType);
        return;
    }
    
    public EditableWSDLBoundPortType getBinding(QName name) {
        return (EditableWSDLBoundPortType)this.bindings.get(name);
    }
    
    public void addService(EditableWSDLService svc) {
        this.services.put(svc.getName(), svc);
    }
    
    public EditableWSDLService getService(QName name) {
        return (EditableWSDLService)this.services.get(name);
    }
    
    public Map<QName, EditableWSDLMessage> getMessages() {
        return this.messages;
    }
    
    @NotNull
    public Map<QName, EditableWSDLPortType> getPortTypes() {
        return this.portTypes;
    }
    
    @NotNull
    public Map<QName, ? extends EditableWSDLBoundPortType> getBindings() {
        return this.unmBindings;
    }
    
    @NotNull
    public Map<QName, EditableWSDLService> getServices() {
        return this.services;
    }
    
    public QName getFirstServiceName() {
        if (this.services.isEmpty()) {
            return null;
        }
        return ((EditableWSDLService)this.services.values().iterator().next()).getName();
    }
    
    public EditableWSDLBoundPortType getBinding(QName serviceName, QName portName) {
        EditableWSDLService service = (EditableWSDLService)this.services.get(serviceName);
        if (service != null) {
            EditableWSDLPort port = service.get(portName);
            if (port != null) {
                return port.getBinding();
            }
        }
        return null;
    }
    
    public void finalizeRpcLitBinding(EditableWSDLBoundPortType boundPortType) {
        assert boundPortType != null;
        QName portTypeName = boundPortType.getPortTypeName();
        if (portTypeName == null) {
            return;
        }
        WSDLPortType pt = (WSDLPortType)this.portTypes.get(portTypeName);
        if (pt == null) {
            return;
        }
        for (EditableWSDLBoundOperation bop : boundPortType.getBindingOperations()) {
            WSDLOperation pto = pt.get(bop.getName().getLocalPart());
            WSDLMessage inMsgName = pto.getInput().getMessage();
            if (inMsgName == null) {
                continue;
            }
            EditableWSDLMessage inMsg = (EditableWSDLMessage)this.messages.get(inMsgName.getName());
            int bodyindex = 0;
            if (inMsg != null) {
                for (EditableWSDLPart part : inMsg.parts()) {
                    String name = part.getName();
                    ParameterBinding pb = bop.getInputBinding(name);
                    if (pb.isBody()) {
                        part.setIndex(bodyindex++);
                        part.setBinding(pb);
                        bop.addPart(part, WebParam.Mode.IN);
                    }
                    else {
                        continue;
                    }
                }
            }
            bodyindex = 0;
            if (pto.isOneWay()) {
                continue;
            }
            WSDLMessage outMsgName = pto.getOutput().getMessage();
            if (outMsgName == null) {
                continue;
            }
            EditableWSDLMessage outMsg = (EditableWSDLMessage)this.messages.get(outMsgName.getName());
            if (outMsg != null) {
                for (EditableWSDLPart part2 : outMsg.parts()) {
                    String name2 = part2.getName();
                    ParameterBinding pb2 = bop.getOutputBinding(name2);
                    if (pb2.isBody()) {
                        part2.setIndex(bodyindex++);
                        part2.setBinding(pb2);
                        bop.addPart(part2, WebParam.Mode.OUT);
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                continue;
            }
        }
        return;
    }
    
    public PolicyMap getPolicyMap() {
        return this.policyMap;
    }
    
    public void setPolicyMap(PolicyMap policyMap) {
        this.policyMap = policyMap;
    }
    
    public void freeze() {
        for (EditableWSDLService service : this.services.values()) {
            service.freeze(this);
        }
        for (EditableWSDLBoundPortType bp : this.bindings.values()) {
            bp.freeze();
        }
        for (EditableWSDLPortType pt : this.portTypes.values()) {
            pt.freeze();
        }
    }
}
